import argparse
import os
import shutil
import subprocess

import requests


def parse_args():
    """Parse control eval arguments."""
    parser = argparse.ArgumentParser(description="Eval ControlNet inside a Docker container")
    parser.add_argument("annotator_type", type=str, help="Type of ControlNet annotator to use")

    return parser.parse_args()


def main(**kwargs):
    """Run the control eval script."""

    pth_name = f"control_sd15_{kwargs['annotator_type']}.pth"

    script_dir = os.path.dirname(os.path.abspath(__file__))

    if not os.path.isfile(os.path.join(script_dir, pth_name)):
        url = f"https://huggingface.co/lllyasviel/ControlNet/resolve/main/models/{pth_name}"
        response = requests.get(url)
        with open(os.path.join(script_dir, pth_name), "wb") as f:
            f.write(response.content)

    if not os.path.isfile(os.path.join(script_dir, "v1-5-pruned.ckpt")):
        url = "https://huggingface.co/runwayml/stable-diffusion-v1-5/resolve/main/v1-5-pruned.ckpt"
        response = requests.get(url)
        with open(os.path.join(script_dir, "v1-5-pruned.ckpt"), "wb") as f:
            f.write(response.content)

    volumes = (
        f"-v {script_dir}/v1-5-pruned.ckpt:/home/engineering/ControlNet/models/v1-5-pruned.ckpt "
        f"-v {script_dir}/{pth_name}:/home/engineering/ControlNet/models/{pth_name}"
    )

    if subprocess.run(["docker", "ps", "-a", "--filter", "name=rb-control"]).returncode == 0:
        subprocess.run(["docker", "stop", "rb-control"])
        subprocess.run(["docker", "rm", "rb-control"])

    if shutil.which("nvidia-smi"):
        subprocess.run(
            ["docker", "run", "--name", "rb-control", "--gpus", "all", "-it", "-d", volumes, "rubbrband/control:latest"]
        )
    else:
        subprocess.run(["docker", "run", "--name", "rb-control", "-it", "-d", volumes, "rubbrband/control:latest"])

    subprocess.run(
        [
            "docker",
            "exec",
            "-it",
            "rb-control",
            "/bin/bash",
            "-c",
            f"conda run --no-capture-output -n control python gradio_{kwargs['annotator_type']}2image.py",
        ]
    )


if __name__ == "__main__":
    main()



import subprocess

import docker
import typer
from yaspin import yaspin

from rubbrband.clients import docker_client
from rubbrband.controllers import eval, train, web
from rubbrband.version import VERSION

__author__ = "Rubbrband"

app = typer.Typer(no_args_is_help=True, rich_markup_mode="rich")
app.add_typer(train.app, name="train", subcommand_metavar="MODEL")
app.add_typer(eval.app, name="eval", subcommand_metavar="MODEL")
app.add_typer(web.app, name="web", subcommand_metavar="MODEL")
image_models = ["lora", "dreambooth", "control"]
webui_models = ["sd-webui"]

try:
    client = docker.from_env()
except docker.errors.DockerException:
    typer.echo("Docker is not running as root. Please start Docker or run sudo su root.")
    exit()


# create our database of models
db = {
    "lora": {
        "description": "Low-rank adaptation for efficient stable diffusion fine-tuning",
        "shape": "anything",
    },
    "dreambooth": {
        "description": "Stable diffusion models, trained with dreambooth method",
        "shape": "anything",
    },
    "control": {
        "description": "Control diffusion models by adding extra conditions",
        "shape": "anything",
    },
    "sd-webui": {
        "description": "Stable diffusion models, trained with webui method",
        "shape": "anything",
    },
}

# Pass singleton objects to our subcommands
train.client = client
eval.client = client
web.client = client
train.db = db
eval.db = db
web.db = db


@app.callback()
def main():
    """
    The Rubbrband CLI allows you to rapidly train and evaluate models.
    """
    pass


@app.command()
def models():
    """List all supported MODELS :robot:"""
    typer.echo("Supported Models:")
    typer.echo(f"{'NAME':12} DESCRIPTION")
    for key, val in db.items():
        typer.echo(f"{key:12} {val['description']}")


@app.command()
def ls():
    """List all running MODELS :robot:"""
    typer.echo("Running Models:")
    containers = client.containers.list()

    # filter containers that start with rb-
    for container in containers:
        if container.name.startswith("rb-"):
            typer.echo(container.name)


@app.command()
def stop(model: str):
    """
    Stop a running MODEL :robot:

    MODEL is the name of the model.

    Example: rubbrband stop lora
    """
    with yaspin() as sp:
        sp.text = "Stopping Docker Container"

        container_name = f"rb-{model}"
        container = docker_client.get_container(container_name)

        if container:
            container.stop()


@app.command()
def copy_to(model: str, src: str, dest: str):
    """
    Copy a file from your computer to a running MODEL :robot:

    MODEL is the name of the model.
    SRC is the path to the file on your computer.
    DEST is the path to the file on the model.

    Example: rubbrband copy-to lora /path/on/my-computer/image.png /path/on/my-model/image.png
    """
    with yaspin() as sp:
        sp.text = "Copying File"

        container_name = f"rb-{model}"
        container = docker_client.get_container(container_name)

        if container:
            subprocess.run(["docker", "cp", src, f"{container.name}:{dest}"])


@app.command()
def copy_from(model: str, src: str, dest: str):
    """
    Copy a file from a running MODEL to your computer :robot:

    MODEL is the name of the model.
    SRC is the path to the file on the model.
    DEST is the path to the file on your computer.

    Example: rubbrband copy-from lora /path/on/my-model/image.png /path/on/my-computer/image.png
    """
    with yaspin() as sp:
        sp.text = "Copying File"

        container_name = f"rb-{model}"
        container = docker_client.get_container(container_name)
        if container:
            subprocess.run(["docker", "cp", f"{container.name}:{src}", dest])


@app.command()
def launch(model: str):
    """
    Launch a new MODEL :robot:

    MODEL is the name of the model to launch.

    Example: rubbrband launch lora
    """

    if model not in db:
        typer.echo("Model not found")
        return

    typer.echo("Downloading model. This may take up to 10 minutes.")
    image_name = f"rubbrband/{model}"
    docker_client.pull_image_handler(image_name)

    if model in webui_models:
        typer.echo("Finished. This model is based on a web interface.")
        typer.echo(
            "When you're ready, run rubbrband web {model} to launch the interface. \
                   We'll let you know how to access it."
        )
    else:
        typer.echo(f"Finished. Run rubbrband train {model} to train this model on sample data.")


@app.command()
def enter(model: str):
    """
    Enter into a running MODEL :robot:

    MODEL is the name of the model.

    Example: rubbrband enter lora
    """
    # if container not running, start it
    with yaspin() as sp:
        sp.text = "Launching Docker Container"

        container_name = f"rb-{model}"
        try:
            container = client.containers.get(container_name)
        except docker.errors.NotFound:
            client.containers.run(container_name, detach=True, name=container_name, tty=True, stdin_open=True)

        container = client.containers.get(container_name)

        if container.status != "running":
            container.start()

    subprocess.run(["docker", "exec", "-it", container_name, "/bin/bash"])


@app.command()
def version():
    """Display the current version of the application"""
    typer.echo(f"Rubbrband CLI version: {VERSION}")


if __name__ == "__main__":
    app()



import argparse
import os
import subprocess
import sys
from datetime import datetime

import GPUtil
import requests


def parse_args():
    """Parse dreambooth train arguments."""

    parser = argparse.ArgumentParser(description="Fine-tune Dreambooth on a dataset with a regularization prompt")
    parser.add_argument(
        "--class_word",
        "-c",
        type=str,
        required=True,
        help="The name that you want to give to the class of images that you'll want to generate",
    )
    parser.add_argument(
        "--dataset_dir",
        "-d",
        type=str,
        required=True,
        help="The full path that contains the images you want to fine-tune on.",
    )
    parser.add_argument(
        "--reg_dir",
        "-r",
        type=str,
        required=True,
        help="The full path that contains the regularization images.",
    )
    parser.add_argument(
        "--model_name",
        "-n",
        type=str,
        required=True,
        help="The name you want to give your model checkpoint file.",
    )
    parser.add_argument(
        "--log_dir",
        "-l",
        type=str,
        required=False,
        help="The full path that contains the directory you want the logs to be in",
    )
    return parser.parse_args()


def check_gpu():
    gpus = GPUtil.getGPUs()

    # Check if there is at least one GPU available
    if len(gpus) > 0:
        # Get the first GPU and check its VRAM
        gpu = gpus[0]
        if gpu.memoryTotal > 25:
            return True
        else:
            print("Graphics card found, but it has less than 25GB VRAM.")
            return False
    else:
        print("No graphics card found.")
        return False


def main(**kwargs):
    """Run the dreambooth train script."""

    script_dir = os.path.dirname(os.path.realpath(__file__))
    ckpt_path = os.path.join(script_dir, "v1-5-pruned.ckpt")

    gpu_ready = check_gpu()

    if not gpu_ready:
        print("You need a GPU with at least 25GB VRAM to fine-tune the model.")
        sys.exit(1)

    if not os.path.isfile(ckpt_path):
        print("Downloading SD checkpoint file. This is about 7GB.")
        url = "https://huggingface.co/runwayml/stable-diffusion-v1-5/resolve/main/v1-5-pruned.ckpt"
        response = requests.get(url)
        if response.status_code == 200:
            with open(ckpt_path, "wb") as f:
                f.write(response.content)
        else:
            print(f"Failed to download checkpoint file. Status code: {response.status_code}")
            sys.exit(1)

    if "rb-dreambooth" in subprocess.check_output(["docker", "ps", "-a"]).decode("utf-8"):
        subprocess.call(["docker", "stop", "rb-dreambooth"])
        subprocess.call(["docker", "rm", "rb-dreambooth"])

    datasetdir = os.path.abspath(kwargs["dataset_dir"])

    # find the token for the dataset directory
    # go inside the dataset directory, and find the names of the directories
    # inside it. The first directory to have another directory called class-name inside it
    # is the token
    token = None
    for dir in os.listdir(datasetdir):
        if os.path.isdir(os.path.join(datasetdir, dir)):
            if os.path.isdir(os.path.join(datasetdir, dir, kwargs["class_word"])):
                token = dir
                break
    if token is None:
        print(
            "The dataset-directory is not in the correct format. \
              Refer to https://rubbrband.gitbook.io/cli-docs/training-models/dreambooth to correct it."
        )
        sys.exit(1)

    regdir = os.path.abspath(kwargs["reg_dir"])
    logdir = os.path.abspath(kwargs["log_dir"])

    subprocess.call(
        [
            "docker",
            "run",
            "--name",
            "rb-dreambooth",
            "--gpus",
            "all",
            "-it",
            "-d",
            "-v",
            os.path.join(script_dir, "v1-5-pruned.ckpt") + ":/home/engineering/v1-5-pruned.ckpt",
            "-v",
            datasetdir + ":/home/engineering/dataset-dir",
            "-v",
            regdir + ":/home/engineering/reg-dir",
            "-v",
            logdir + ":/home/engineering/log-dir",
            "-d",
            "rubbrband/dreambooth:latest",
        ]
    )

    class_word = kwargs["class_word"]
    model_name = kwargs["model_name"]

    # count the number of images in dataset_dir, including files in subdirectories
    num_images = sum(len(files) for _, _, files in os.walk(kwargs["dataset_dir"]))
    training_steps = num_images * 70

    if num_images < 50:
        print("You should have least 100 images to finetune the model. Otherwise, you may not get good results")

    # get current datetime in UTC format
    now_utc = datetime.utcnow()

    # create log file name with timestamp
    log_file_name = f"logfile-out{now_utc.strftime('%Y-%m-%dT%H-%M-%SZ')}_{model_name}.log"

    # create log directory inside the docker container
    subprocess.call("docker exec rb-dreambooth /bin/bash -c 'mkdir -p /home/engineering/log-dir/'", shell=True)

    conda_cmd = (
        "conda run --no-capture-output -n ldm",
        "python /home/engineering/JoePenna-Dreambooth/main.py "
        "--base /home/engineering/JoePenna-Dreambooth/configs/stable-diffusion/v1-finetune_unfrozen.yaml",
        f"-t --actual_resume /home/engineering/v1-5-pruned.ckpt -n {model_name} --gpus 0,",
        f"--token {token} --class_word {class_word} --max_training_steps {training_steps} --no-test",
        "--data_root /home/engineering/dataset-dir --reg_data_root /home/engineering/reg-dir",
        "--logdir /home/engineering/log-dir",
    )

    docker_cmd = f"docker exec -it rb-dreambooth /bin/bash -c \"{' '.join(conda_cmd)}\""

    # run the docker container and save the output to a log file
    subprocess.call(f"script -c '{docker_cmd}' {logdir}/{log_file_name}", shell=True)


if __name__ == "__main__":
    main()



import argparse
import subprocess


def parse_args():
    """Parse lora eval arguments."""
    parser = argparse.ArgumentParser(description="Eval lora inside a Docker container")
    parser.add_argument("-i", "--input_prompt", required=True, help="Input prompt passed to your fine-tuned model")
    return parser.parse_args()


def main(**kwargs):
    """Run the lora eval script."""

    # Call the shell command
    command = ["docker", "exec", "-it", "rb-lora", "python", "/home/engineering/infer.py", kwargs["input_prompt"]]
    subprocess.call(command)


if __name__ == "__main__":
    main()



import docker
import typer
from rich.progress import Progress

tasks = {}

try:
    DOCKER_CLIENT = docker.from_env()
except docker.errors.DockerException:
    typer.echo("Docker is not running as root. Please start Docker or run sudo su root.")
    exit()


def show_progress(line, progress):
    """Show progress of a docker pull (red for download, green for extract)"""
    if line["status"] == "Downloading":
        id = f'[red][Download {line["id"]}]'
    elif line["status"] == "Extracting":
        id = f'[green][Extract {line["id"]}]'
    else:
        # skip other statuses
        return

    if id not in tasks:
        tasks[id] = progress.add_task(f"{id}", total=line["progressDetail"]["total"])
    else:
        progress.update(tasks[id], completed=line["progressDetail"]["current"])

    # remove the download task if complete
    download_id = f'[red][Download {line["id"]}]'
    if line["status"] == "Extracting" and download_id in tasks:
        progress.remove_task(tasks[download_id])
        del tasks[download_id]


def pull_image(image_name):
    with Progress() as progress:
        resp = DOCKER_CLIENT.api.pull(image_name, stream=True, decode=True)
        for line in resp:
            show_progress(line, progress)


def pull_image_handler(image_name):
    """Pulls an image from the registry if it is not found locally or if the local image is outdated."""

    matched_images = DOCKER_CLIENT.images.list(image_name)
    registry_digest_sha = DOCKER_CLIENT.images.get_registry_data(image_name).attrs["Descriptor"]["digest"]

    try:
        local_digest_sha = matched_images[0].attrs["RepoDigests"][0].split("@")[1]
    except IndexError:
        local_digest_sha = None

    if len(matched_images) == 0:
        typer.echo("Model not found locally, downloading model.")
        pull_image(image_name)
    elif local_digest_sha != registry_digest_sha:
        typer.echo("Model found locally, but is outdated. Downloading new model.")
        pull_image(image_name)
    else:
        typer.echo("Model found locally, skipping download.")


def get_container(container_name):
    """Returns a container object if it exists and is running."""
    try:
        container = DOCKER_CLIENT.containers.get(container_name)
    except docker.errors.NotFound:
        typer.echo("Model not found, try running rubbrband launch <model>")
        return

    if container.status != "running":
        typer.echo("Model is not running, attempting to start model.")
        try:
            container.start()
        except docker.errors.APIError:
            typer.echo("Unable to start model. Please try again.")
            return

    return container
